﻿// (c) IMT - Information Management Technology AG, CH-9470 Buchs, www.imt.ch. 
// SW guideline: SOP-Coding Guidelines Ver. 1.6
// Autor: Willener

using imtanalytics.ImtUsbProtocol;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Management;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;

namespace ImtUsbProtcolTestApp.Device {

    /// <summary>
    /// Citrex H5 device connection handling.
    /// </summary>
    public class CitrexConnection : INotifyPropertyChanged {

        /// <summary>
        /// Property change event.
        /// </summary>
        public event PropertyChangedEventHandler? PropertyChanged;

        public Citrex Citrex {
            get { return m_Citrex; }
            set { m_Citrex = value;
                NotifyPropertyChanged("Citrex");
            }
        }

        /// <summary>
        /// Load the PF300Pro and CITREX device USB device connections from the computer.
        /// </summary>
        public static IEnumerable<string> GetCitrexDevices() {
            try {
                var thread = new Thread(() => {
                    m_citrexDevices = DevicePorts.GeCitrextDevices();
                });
                thread.Start();
                thread.Join();
            }
            catch (Exception ex) {
                Debug.WriteLine(ex.Message);
            }

            return m_citrexDevices;
        }

        /// <summary>
        /// Connect to the available Citrex H5 devices.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public static Citrex Connect() {
            GetCitrexDevices();
            foreach (var device in m_citrexDevices) {
                if (m_Citrex != null &&
                   !m_Citrex.IsConnected) {
                    m_Citrex.Connect(device);
                }
            }
            return m_Citrex;
        }

        /// <summary>
        /// Disconnect all active connections.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public static void Disconnect() {
            try {
                if (m_Citrex != null && m_Citrex.IsConnected) {
                    m_Citrex.Disconnect();
                    Thread.Sleep(5000);
                }
            }
            catch (Exception ex) {
                Debug.WriteLine(ex.Message);
            }
        }

        /// <summary>
        /// Event handler for property changed
        /// </summary>
        /// <param name="propertyName"></param>
        protected virtual void NotifyPropertyChanged(string propertyName) {
            PropertyChangedEventHandler? handler = PropertyChanged;
            if (handler != null) {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        private static IEnumerable<string> m_citrexDevices = new List<string>();
        private static Citrex m_Citrex = new Citrex();
    }
}
