﻿// (c) IMT - Information Management Technology AG, CH-9470 Buchs, www.imt.ch. 
// SW guideline: SOP-Coding Guidelines Ver. 1.6
// Autor: Willener

using imtanalytics.ImtUsbProtocol.Device.Citrex.RemoteObjects;
using imtanalytics.ImtUsbProtocol.EnumIdentifiers;
using ImtUsbProtocol.Citrex;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using static System.Net.Mime.MediaTypeNames;

namespace ImtUsbProtcolTestApp.Device {

    /// <summary>
    /// Citrex H5 device settings
    /// </summary>
    public class CitrexSettings : INotifyPropertyChanged {
        
        /// <summary>
        /// Property change event.
        /// </summary>
        public event PropertyChangedEventHandler? PropertyChanged;

        public IEnumerable<string> CitrexAnalogOut1ValueModelId {
            get {
                return Enum.GetNames(typeof(AnalogOutValueModelId));
            }
        }

        public IEnumerable<string> CitrexAnalogOut2ValueModelId {
            get {
                return Enum.GetNames(typeof(AnalogOutValueModelId));
            }
        }

        public IEnumerable<string> CitrexEthernetIpMode {
            get {
                return Enum.GetNames(typeof(EthernetIpMode));
            }
        }

        /// <summary>
        /// Citrex H5 <see cref="CitrexVentilationParameterPressureSource"/>.
        /// </summary>
        public IEnumerable<string> CitrexVentilationParameterPressureSource {
            get {
                return Enum.GetNames(typeof(CitrexVentilationParameterPressureSource));
            }
        }

        /// <summary>
        /// Citrex H5 <see cref="FilterType"/>.
        /// </summary>
        public IEnumerable<string> CitrexFilterType {
            get {
                return Enum.GetNames(typeof(FilterType));
            }
        }

        /// <summary>
        /// Citrex H5 <see cref="VolumeStandard"/>.
        /// </summary>
        public IEnumerable<string> CitrexVolumeStandard {
            get {
                return Enum.GetNames(typeof(VolumeStandard));
            }
        }

        /// <summary>
        /// Citrex H5 <see cref="GasType"/>.
        /// </summary>
        public IEnumerable<string> CitrexGasType {
            get {
                return Enum.GetNames(typeof(CitrexGasType));
            }
        }

        /// <summary>
        /// Citrex H5 <see cref="SensorPressureCompensationType"/>.
        /// </summary>
        public IEnumerable<string> CitrexPressureCompensationType {
            get {
                return Enum.GetNames(typeof(SensorPressureCompensationType));
            }
        }

        /// <summary>
        /// Citrex H5 <see cref="CitrexRS232Protocol"/>.
        /// </summary>
        public IEnumerable<string> CitrexRS232Protocol {
            get {
                return Enum.GetNames(typeof(CitrexRS232Protocol));
            }
        }

        /// <summary>
        /// Citrex H5 <see cref="TriggerMode"/>.
        /// </summary>
        public IEnumerable<string> CitrexTriggerMode {
            get {
                return Enum.GetNames(typeof(TriggerMode));
            }
        }

        /// <summary>
        /// Citrex H5 <see cref="TriggerSource"/>.
        /// </summary>
        public IEnumerable<string> CitrexTriggerSource {
            get {
                return Enum.GetNames(typeof(TriggerSource));
            }
        }

        /// <summary>
        /// Citrex H5 <see cref="TriggerEdge"/>.
        /// </summary>
        public IEnumerable<string> CitrexTriggerStartEdge {
            get {
                return Enum.GetNames(typeof(TriggerEdge));
            }
        }

        /// <summary>
        /// Citrex H5 <see cref="TriggerSignal"/>.
        /// </summary>
        public IEnumerable<string> CitrexTriggerStartSignal {
            get {
                return Enum.GetNames(typeof(TriggerSignal));
            }
        }

        /// <summary>
        /// Citrex H5 <see cref="TriggerEdge"/>.
        /// </summary>
        public IEnumerable<string> CitrexTriggerEndEdge {
            get {
                return Enum.GetNames(typeof(TriggerEdge));
            }
        }

        /// <summary>
        /// Citrex H5 <see cref="TriggerSignal"/>.
        /// </summary>
        public IEnumerable<string> CitrexTriggerEndSignal {
            get {
                return Enum.GetNames(typeof(TriggerSignal));
            }
        }

        /// <summary>
        /// Citrex H5 <see cref="TriggerSignal"/>.
        /// </summary>
        public Visibility IsVisibleCitrexTriggerSetting {
            get {
                return m_IsVisibleCitrexTriggerSetting;
            }
            set {
                m_IsVisibleCitrexTriggerSetting = value;
                NotifyPropertyChanged("IsVisibleCitrexTriggerSetting");
            }
        }

        /// <summary>
        /// Citrex H5 <see cref="TriggerConfigRemoteObject"/>.
        /// </summary>
        public TriggerConfigRemoteObject? CitrexTriggerConfigRemoteObject {
            get {
                return m_CitrexTriggerSettingsRemoteObject;
            }
            set {
                if (m_CitrexTriggerSettingsRemoteObject != value) {
                    m_CitrexTriggerSettingsRemoteObject = value;
                    NotifyPropertyChanged("CitrexTriggerSettingsRemoteObject");
                }
            }
        }

        /// <summary>
        /// Citrex H5 <see cref="DeviceConfigRemoteObject"/>.
        /// </summary>
        public DeviceConfigRemoteObject? CitrexDeviceConfigRemoteObject {
            get {
                return m_CitrexDeviceConfigRemoteObject;
            }
            set {
                if (m_CitrexDeviceConfigRemoteObject != value) {
                    m_CitrexDeviceConfigRemoteObject = value;
                    NotifyPropertyChanged("CitrexDeviceConfigRemoteObject");
                }
            }
        }

        /// <summary>
        /// Citrex H5 <see cref="CitrexAllData"/>.
        /// </summary>
        public CitrexAllData CitrexAllData {
            get {
                return m_CitrexAllData;
            }
            set {
                if (m_CitrexAllData != value) {
                    m_CitrexAllData = value;
                    NotifyPropertyChanged("CitrexAllData");
                }
            }
        }

        /// <summary>
        /// Citrex H5 <see cref="CitrexFastData"/>.
        /// </summary>
        public CitrexFastData CitrexFastData {
            get {
                return m_CitrexFastData;
            }
            set {
                if (m_CitrexFastData != value) {
                    m_CitrexFastData = value;
                    NotifyPropertyChanged("CitrexFastData");
                }
            }
        }

        /// <summary>
        /// Event handler for property changed
        /// </summary>
        /// <param name="propertyName"></param>
        protected virtual void NotifyPropertyChanged(string propertyName) {
            PropertyChangedEventHandler? handler = PropertyChanged;
            if (handler != null) {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        private CitrexAllData m_CitrexAllData = new CitrexAllData();
        private CitrexFastData m_CitrexFastData = new CitrexFastData();
        private TriggerConfigRemoteObject? m_CitrexTriggerSettingsRemoteObject;
        private DeviceConfigRemoteObject? m_CitrexDeviceConfigRemoteObject;

        private Visibility m_IsVisibleCitrexTriggerSetting = Visibility.Visible;
    }
}
