﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Management;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace ImtUsbProtcolTestApp.Device {

    /// <summary>
    /// Search for connected Citrex and Pf300Pro USB devices.
    /// </summary>
    public class DevicePorts {

        /// <summary>
        /// Load the PF300Pro device USB device connections from the computer.
        /// </summary>
        public static IEnumerable<string> GetPf300ProDevices() {
            var pf300ProDevices = new List<string>();
            var scope = new ManagementScope("root\\CIMV2") { Options = { EnablePrivileges = true } };
            const string win32PnPEntity = "Select * From Win32_PnPEntity WHERE Caption LIKE '%Serial Port%'";
            var query = new ObjectQuery(win32PnPEntity);
            var mySearcher = new ManagementObjectSearcher(scope, query);
            foreach (ManagementObject managementObject in mySearcher.Get().Cast<ManagementObject>()) {
                if (managementObject["DeviceID"] == null) {
                    continue;
                }
                var deviceId = managementObject["DeviceID"].ToString();
                if (deviceId == null) {
                    continue;
                }
                string[] dataId = deviceId.Split('\\')[1].Split('+');
                if (dataId.Length <= 1) {
                    continue;
                }
                if (managementObject["Manufacturer"] == null) {
                    continue;
                }
                var manufacturerObeject = managementObject["Manufacturer"].ToString();
                if (manufacturerObeject == null) {
                    continue;
                }
                string manufacturer = manufacturerObeject;
                string vendorId = dataId[0].Substring(4, 4);
                string productId = dataId[1].Substring(4, 4);
                string? caption = managementObject["Caption"].ToString();
                if (caption == null) {
                    continue;
                }
                int startIndex = caption.IndexOf("(", StringComparison.Ordinal);
                string comPort = caption.Substring(startIndex + 1, caption.Length - startIndex - 2);
                //Filter for device description
                if (VENDOR_ID != vendorId ||
                    PRODUCT_ID != productId) {
                    if (manufacturer != "Mobile Gas Flow Analyzer CDC Device") {
                        continue;
                    }
                }

                pf300ProDevices.Add(comPort);
                break;
            }

            return pf300ProDevices;
        }


        /// <summary>
        /// Load the CITREX device USB device connections from the computer.
        /// </summary>
        public static IEnumerable<string> GeCitrextDevices() {
            var citrexDevices = new List<string>();
            var scope = new ManagementScope("root\\CIMV2") { Options = { EnablePrivileges = true } };
            const string win32PnPEntity = "SELECT * FROM Win32_PnPEntity WHERE Caption like '%Mobile Gas Flow Analyzer CDC Device%'";
            var query = new ObjectQuery(win32PnPEntity);
            var mySearcher = new ManagementObjectSearcher(scope, query);
            foreach (ManagementObject managementObject in mySearcher.Get().Cast<ManagementObject>()) {
                if (managementObject == null || managementObject["Caption"] == null) {
                    continue;
                }
                string? fullName = managementObject["Caption"] as string;
                if (fullName == null) { continue; }
                Match m = Regex.Match(fullName, @".*\((COM\d+)\)$");
                // Note that the groups always have at least one empty object so the actual count is captured groups + 1
                if (m != null && m.Groups.Count == 2) {
                    string comPort = m.Groups[1].Value;
                    if (!String.IsNullOrEmpty(comPort)) {
                        citrexDevices.Add(comPort);
                    }
                }
                break;
            }

            return citrexDevices;
        }


        private const string VENDOR_ID = "0403";
        private const string PRODUCT_ID = "6014";
    }
}
