﻿// (c) IMT - Information Management Technology AG, CH-9470 Buchs, www.imt.ch. 
// SW guideline: SOP-Coding Guidelines Ver. 1.6
// Autor: Willener

using imtanalytics.ImtUsbProtocol.Device.PF300Pro.RemoteObjects;
using imtanalytics.ImtUsbProtocol.EnumIdentifiers;
using ImtUsbProtcolTestApp.Device;
using ImtUsbProtocol.PF300Pro;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows;

namespace ImtUsbProtcolTestApp {

    /// <summary>
    /// Device settings summary
    /// </summary>
    public class DeviceSettings : INotifyPropertyChanged {

        /// <summary>
        /// Property change event.
        /// </summary>
        public event PropertyChangedEventHandler? PropertyChanged;

        /// <summary>
        /// All available the Citrex H5 settings.
        /// </summary>
        public CitrexSettings CitrexSettings {
            get {
                return m_CitrexSettings;
            }
            set {
                m_CitrexSettings = value;
                NotifyPropertyChanged("CitrexSettings");
            }
        }

        /// <summary>
        /// All available the PF300Pro settings.
        /// </summary>
        public Pf300ProSettings Pf300ProSettings {
            get {
                return m_Pf300ProSettings;
            }
            set {
                m_Pf300ProSettings = value;
                NotifyPropertyChanged("Pf300ProSettings");
            }
        }

        /// <summary>
        /// Event handler for property changed
        /// </summary>
        /// <param name="propertyName"></param>
        protected virtual void NotifyPropertyChanged(string propertyName) {
            PropertyChangedEventHandler? handler = PropertyChanged;
            if (handler != null) {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        /// <summary>
        /// Constructor
        /// </summary>
        public DeviceSettings() {
            Pf300ProSettings = new Pf300ProSettings();
            CitrexSettings = new CitrexSettings();
        }

        private CitrexSettings m_CitrexSettings = new CitrexSettings();
        private Pf300ProSettings m_Pf300ProSettings = new Pf300ProSettings();
    }
}