﻿// (c) IMT - Information Management Technology AG, CH-9470 Buchs, www.imt.ch. 
// SW guideline: SOP-Coding Guidelines Ver. 1.6
// Autor: Willener

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using imtanalytics.ImtUsbProtocol;

namespace ImtUsbProtcolTestApp.Device {

    /// <summary>
    /// Pf300Pro device connection handling.
    /// </summary>
    public  class Pf300ProConnection : INotifyPropertyChanged {

        /// <summary>
        /// Property change event.
        /// </summary>
        public event PropertyChangedEventHandler? PropertyChanged;

        public FlowAnalyserPro FlowAnalyserPro {
            get { return m_Pf300Pro; }
            set {
                m_Pf300Pro = value;
                NotifyPropertyChanged("FlowAnalyserPro");
            }
        }

        /// <summary>
        /// Load the PF300Pro and CITREX device USB device connections from the computer.
        /// </summary>
        public static IEnumerable<string> GetPf300ProDevices() {
            try {
                var thread = new Thread(() => {
                    m_pf300ProDevices = DevicePorts.GetPf300ProDevices();
                });
                thread.Start();
                thread.Join();
            }
            catch (Exception ex) {
                Debug.WriteLine(ex.Message);
            }

            return m_pf300ProDevices;
        }

        /// <summary>
        /// Connect to the available PF300Pro devices.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public static FlowAnalyserPro Connect() {
            GetPf300ProDevices();
            foreach (var device in m_pf300ProDevices) {
                if (m_Pf300Pro != null &&
                    !m_Pf300Pro.IsConnected) {
                    m_Pf300Pro.Connect(device);
                }
            }

            return m_Pf300Pro;
        }

        /// <summary>
        /// Disconnect all active connections.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public static void Disconnect() {
            try {
                if (m_Pf300Pro != null && m_Pf300Pro.IsConnected) {
                    m_Pf300Pro.Disconnect();
                }
            }
            catch (Exception ex) {
                Debug.WriteLine(ex.Message);
            }
        }

        /// <summary>
        /// Event handler for property changed
        /// </summary>
        /// <param name="propertyName"></param>
        protected virtual void NotifyPropertyChanged(string propertyName) {
            PropertyChangedEventHandler? handler = PropertyChanged;
            if (handler != null) {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        private static IEnumerable<string> m_pf300ProDevices = new List<string>();
        private static FlowAnalyserPro m_Pf300Pro = new FlowAnalyserPro();
    }
}
