﻿// (c) IMT - Information Management Technology AG, CH-9470 Buchs, www.imt.ch. 
// SW guideline: SOP-Coding Guidelines Ver. 1.6
// Autor: Willener

using imtanalytics.ImtUsbProtocol.Device.PF300Pro.RemoteObjects;
using imtanalytics.ImtUsbProtocol.EnumIdentifiers;
using ImtUsbProtocol.PF300Pro;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace ImtUsbProtcolTestApp.Device {

    /// <summary>
    /// PF300Pro device settings
    /// </summary>
    public class Pf300ProSettings: INotifyPropertyChanged {

        /// <summary>
        /// Property change event.
        /// </summary>
        public event PropertyChangedEventHandler? PropertyChanged;

        /// <summary>
        /// PF300Pro <see cref="BaseFlowMode"/>.
        /// </summary>
        public IEnumerable<string> DeviceBaseFlowMode {
            get {
                return Enum.GetNames(typeof(BaseFlowMode));
            }
        }

        /// <summary>
        /// PF300Pro <see cref="GasType"/>.
        /// </summary>
        public IEnumerable<string> DeviceGasType {
            get {
                return Enum.GetNames(typeof(Pf300ProGasType));
            }
        }

        /// <summary>
        /// PF300Pro <see cref="VolumeStandard"/>.
        /// </summary>
        public IEnumerable<string> DeviceVolumeStandard {
            get {
                return Enum.GetNames(typeof(VolumeStandard));
            }
        }

        /// <summary>
        /// PF300Pro <see cref="UlfVolumeStandard"/>.
        /// </summary>
        public IEnumerable<string> DeviceUlfVolumeStandard {
            get {
                return Enum.GetNames(typeof(UlfVolumeStandard));
            }
        }

        /// <summary>
        /// PF300Pro <see cref="UlfGasType"/>.
        /// </summary>
        public IEnumerable<string> DeviceUlfGasType {
            get {
                return Enum.GetNames(typeof(UlfGasType));
            }
        }

        /// <summary>
        /// PF300Pro <see cref="DeviceVentilationParameterPressureSource"/>.
        /// </summary>
        public IEnumerable<string> DeviceVentilationParameterPressureSource {
            get {
                return Enum.GetNames(typeof(Pf300ProVentilationParameterPressureSource));
            }
        }

        /// <summary>
        /// PF300Pro <see cref="FilterType"/>.
        /// </summary>
        public IEnumerable<string> DeviceFilterType {
            get {
                return Enum.GetNames(typeof(FilterType));
            }
        }

        /// <summary>
        /// PF300Pro <see cref="Pf300ProRS232Protocol"/>.
        /// </summary>
        public IEnumerable<string> DeviceRS232Protocol {
            get {
                return Enum.GetNames(typeof(Pf300ProRS232Protocol));
            }
        }

        /// <summary>
        /// PF300Pro <see cref="TriggerMode"/>.
        /// </summary>
        public IEnumerable<string> DeviceTriggerMode {
            get {
                return Enum.GetNames(typeof(TriggerMode));
            }
        }

        /// <summary>
        /// PF300Pro <see cref="TriggerSource"/>.
        /// </summary>
        public IEnumerable<string> DeviceTriggerSource {
            get {
                return Enum.GetNames(typeof(TriggerSource));
            }
        }

        /// <summary>
        /// PF300Pro <see cref="TriggerEdge"/>.
        /// </summary>
        public IEnumerable<string> DeviceTriggerStartEdge {
            get {
                return Enum.GetNames(typeof(TriggerEdge));
            }
        }

        /// <summary>
        /// PF300Pro <see cref="TriggerSignal"/>.
        /// </summary>
        public IEnumerable<string> DeviceTriggerStartSignal {
            get {
                return Enum.GetNames(typeof(TriggerSignal));
            }
        }

        /// <summary>
        /// PF300Pro <see cref="TriggerEdge"/>.
        /// </summary>
        public IEnumerable<string> DeviceTriggerEndEdge {
            get {
                return Enum.GetNames(typeof(TriggerEdge));
            }
        }

        /// <summary>
        /// PF300Pro <see cref="TriggerSignal"/>.
        /// </summary>
        public IEnumerable<string> DeviceTriggerEndSignal {
            get {
                return Enum.GetNames(typeof(TriggerSignal));
            }
        }

        /// <summary>
        /// PF300Pro if the TriggerLevel and Signal is visible <see cref="Visibility"/>.
        /// </summary>
        public Visibility IsVisibleTriggerSetting {
            get {
                return m_IsVisibleTriggerSetting;
            }
            set {
                m_IsVisibleTriggerSetting = value;
                NotifyPropertyChanged("IsVisibleTriggerSetting");
            }
        }

        /// <summary>
        /// PF300Pro <see cref="DeviceSettingsRemoteObject"/>.
        /// </summary>
        public DeviceSettingsRemoteObject? DeviceSettingsRemoteObject {
            get {
                return m_DeviceSettingsRemoteObject;
            }
            set {
                if (m_DeviceSettingsRemoteObject != value) {
                    m_DeviceSettingsRemoteObject = value;
                    NotifyPropertyChanged("DeviceSettingsRemoteObject");
                }
            }
        }

        /// <summary>
        /// PF300Pro <see cref="TriggerSettingsRemoteObject"/>.
        /// </summary>
        public TriggerSettingsRemoteObject? TriggerSettingsRemoteObject {
            get {
                return m_TriggerSettingsRemoteObject;
            }
            set {
                if (m_TriggerSettingsRemoteObject != value) {
                    m_TriggerSettingsRemoteObject = value;
                    NotifyPropertyChanged("TriggerSettingsRemoteObject");
                }
            }
        }

        /// <summary>
        /// PF300Pro <see cref="MeasurementSettingsRemoteObject"/>.
        /// </summary>
        public MeasurementSettingsRemoteObject? MeasurementSettingsRemoteObject {
            get {
                return m_MeasurementSettingsRemoteObject;
            }
            set {
                if (m_MeasurementSettingsRemoteObject != value) {
                    m_MeasurementSettingsRemoteObject = value;
                    NotifyPropertyChanged("MeasurementSettingsRemoteObject");
                }
            }
        }

        /// <summary>
        /// PF300Pro <see cref="PF300ProAllData"/>.
        /// </summary>
        public PF300ProAllData PF300ProAllData {
            get {
                return m_PF300ProAllData;
            }
            set {
                if (m_PF300ProAllData != value) {
                    m_PF300ProAllData = value;
                    NotifyPropertyChanged("PF300ProAllData");
                }
            }
        }

        /// <summary>
        /// PF300Pro <see cref="PF300ProFastData"/>.
        /// </summary>
        public PF300ProFastData PF300ProFastData {
            get {
                return m_PF300ProFastData;
            }
            set {
                if (m_PF300ProFastData != value) {
                    m_PF300ProFastData = value;
                    NotifyPropertyChanged("PF300ProFastData");
                }
            }
        }

        /// <summary>
        /// PF300Pro <see cref="PF300ProFastData"/>.
        /// </summary>
        public PF300ProIrmaData PF300ProIrmaData {
            get {
                return m_PF300ProIrmaData;
            }
            set {
                if (m_PF300ProIrmaData != value) {
                    m_PF300ProIrmaData = value;
                    NotifyPropertyChanged("PF300ProIrmaData");
                }
            }
        }

        /// <summary>
        /// Event handler for property changed
        /// </summary>
        /// <param name="propertyName"></param>
        protected virtual void NotifyPropertyChanged(string propertyName) {
            PropertyChangedEventHandler? handler = PropertyChanged;
            if (handler != null) {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        private PF300ProAllData m_PF300ProAllData = new PF300ProAllData();
        private PF300ProFastData m_PF300ProFastData = new PF300ProFastData();
        private PF300ProIrmaData m_PF300ProIrmaData = new PF300ProIrmaData();

        private MeasurementSettingsRemoteObject? m_MeasurementSettingsRemoteObject;
        private TriggerSettingsRemoteObject? m_TriggerSettingsRemoteObject;
        private DeviceSettingsRemoteObject? m_DeviceSettingsRemoteObject;

        private Visibility m_IsVisibleTriggerSetting = Visibility.Visible;
    }
}